#ifndef CTOR_EXCEPTION_H_INCLUDE
#define CTOR_EXCEPTION_H_INCLUDE

#include <iostream>
#include <exception>
#include <stdexcept>
#include <string>

struct Foo
{
	typedef Foo* Ptr;
	std::string name_;
	Foo(std::string const& n) : name_(n)
	{ std::cout << "   " << name_ << ": Foo::Foo()\n"; }
	~Foo() { std::cout << "   " << name_ << ": Foo::~Foo()\n"; }
};

struct A
{
	Foo::Ptr unsafe_;
	std::string name_;

	A(std::string const& name, bool except, Foo::Ptr foo)
		: name_(name), unsafe_(foo)
	{
		std::cout << name_ << ": A::A()\n";

		if (except)
		{
			std::cout << name_ << ": ctor says hello but dctor won't have any chance to say it!\n";
			throw std::logic_error("We are NOT cool!");
		}
	}

	~A()
	{
		std::cout << name_ << ": A::~A()\n";
		delete unsafe_; // We are cool!
	}
};

void test_ctor_exception()
{
	std::cout << "------ test_ctor_exception - start ------\n";
	try
	{
		Foo::Ptr noleak = new Foo("noleak");
		A a("not-throwing", false, noleak);
	}
	catch (std::exception const& e)
	{
		std::cout << e.what() << std::endl;
	}

	std::cout << std::endl;
	
	try
	{
		Foo::Ptr bigleak = new Foo("bigleak");
		A a("throwing", true, bigleak);
	}
	catch (std::exception const& e)
	{
		std::cout << e.what() << std::endl;
	}
	std::cout << "------ test_ctor_exception - end ------\n";
}

#endif
