#ifndef CTOR_EXCEPTION_SMART_PTR_H_INCLUDE
#define CTOR_EXCEPTION_SMART_PTR_H_INCLUDE

#include <iostream>
#include <exception>
#include <stdexcept>
#include <string>
#include <boost/shared_ptr.hpp>

struct Bar
{
	typedef boost::shared_ptr<Bar> Ptr;
	std::string name_;
	Bar(std::string const& n) : name_(n)
	{ std::cout << "   " << name_ << ": Bar::Bar()\n"; }
	~Bar() { std::cout << "   " << name_ << ": Bar::~Bar()\n"; }
};

struct B
{
	Bar::Ptr safe_;
	std::string name_;

	B(std::string const& name, bool except, Bar::Ptr foo)
		: name_(name), safe_(foo)
	{
		std::cout << name_ << ": B::B()\n";

		if (except)
		{
			std::cout << name_ << ": ctor says hello AND dctors of B and Bar also say hello!\n";
			throw std::logic_error("We are VERY cool!");
		}
	}

	~B()
	{
		std::cout << name_ << ": B::~B()\n";
		// We are cool!
	}
};

void test_ctor_exception_smart_ptr()
{
	std::cout << "------ test_ctor_exception_smart_ptr - start ------\n";
	try
	{
		Bar::Ptr noleak1(new Bar("noleak1"));
		B a("not-throwing", false, noleak1);
	}
	catch (std::exception const& e)
	{
		std::cout << e.what() << std::endl;
	}
	
	std::cout << std::endl;

	try
	{
		Bar::Ptr noleak2(new Bar("noleak2"));
		B a("throwing", true, noleak2);
	}
	catch (std::exception const& e)
	{
		std::cout << e.what() << std::endl;
	}

	std::cout << "------ test_ctor_exception_smart_ptr - end ------\n";
}

#endif
